<?php

namespace Drupal\media_icon_deliver\Controller;

use Drupal\Core\Cache\CacheableResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\file\Entity\File;
use Drupal\file\FileInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns the SVG markup for icon media entities.
 */
class IconController extends ControllerBase {

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;


  /**
   * The file name.
   *
   * @var string
   */
  protected $fieldName = 'field_media_svg';

  /**
   * Constructs a new instance of the class.
   *
   * @param EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager object.
   * @param ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ConfigFactoryInterface $configFactory) {
    $this->entityTypeManager = $entityTypeManager;
    $this->config = $configFactory->get('media_icon_deliver.settings');

    if ($field_name = $this->config->get('field_name')) {
      $this->fieldName = $field_name;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('config.factory')
    );
  }

  /**
   * Build the response.
   *
   * @param string $mid
   *   The media ID.
   *
   * @return CacheableResponse
   *   Icon response.
   */
  public function build(string $mid): CacheableResponse {
    $response = new CacheableResponse();
    $mediaStorage = $this->entityTypeManager->getStorage('media');
    $media = $mediaStorage->load($mid);

    // Intentional assignment to file variable.
    if ($media && $media->bundle() === 'icon' && $file = $this->getIcon($media)) {
      // The SVG are small and so we don't need to worry about streaming the file
      // contents with a binary file response.
      $content = file_get_contents($file->getFileUri());
      if ($content !== FALSE) {
        $headers = file_get_content_headers($file);
        $response->headers->add($headers);
        return $response
          ->setContent($content)
          ->addCacheableDependency($media)
          ->addCacheableDependency($file);
      }
    }

    // Return an empty response so that we don't render a full fledged Drupal
    // error page.
    $response->setStatusCode(404);
    $response->setContent('');
    return $response;
  }

  private function getIcon($media) {
    // Should never occur as the field is mandatory.
    if ($media->get($this->fieldName)->isEmpty()) {
      return NULL;
    }
    $file = $media->get($this->fieldName)->entity;
    if (!$file instanceof FileInterface) {
      return NULL;
    }
    assert($file instanceof File);

    return $file;
  }

}
